#ifndef GARMENT_H
#define GARMENT_H

#include <Arduino.h>

class Garment {
 public:
  String type;
  String name;
  int minTemp;
  int maxTemp;
  int added_warmth;
  bool weather_proof;

  Garment(String t, String n, int min, int max, int warmth, bool weather_proof)
      : type(t),
        name(n),
        minTemp(min),
        maxTemp(max),
        added_warmth(warmth),
        weather_proof(weather_proof) {}

  bool isSuitable(int temp) { return temp >= minTemp && temp <= maxTemp; }
  String getName() { return name; }
};

class Headwear : public Garment {
 public:
  Headwear(String n, int min, int max, int warmth, bool weather_proof)
      : Garment("Headwear", n, min, max, warmth, weather_proof) {}
};

class Top : public Garment {
 public:
  Top(String n, int min, int max, int warmth, bool weather_proof)
      : Garment("Top", n, min, max, warmth, weather_proof) {}
};

class Pants : public Garment {
 public:
  Pants(String n, int min, int max, int warmth, bool weather_proof)
      : Garment("Pants", n, min, max, warmth, weather_proof) {}
};

class Shoes : public Garment {
 public:
  Shoes(String n, int min, int max, int warmth, bool weather_proof)
      : Garment("Shoes", n, min, max, warmth, weather_proof) {}
};

class Outfit {
 public:
  Headwear* headwear;
  Top* tops[5];  // Max 5 tops
  int topsCount;
  int topsWarmth;
  Pants* pants;
  Shoes* shoes;

  Outfit()
      : headwear(nullptr),
        pants(nullptr),
        shoes(nullptr),
        topsCount(0),
        topsWarmth(0) {}

  Outfit(Headwear* h, Pants* p, Shoes* s)
      : headwear(h), pants(p), shoes(s), topsCount(0) {}

  void addTop(Top* t) {
    if (topsCount < 5) {
      tops[topsCount] = t;
      topsCount++;
      topsWarmth += t->added_warmth;
    }
  }

  String toString() {
    String outfitString;
    outfitString += "Headwear: " + headwear->getName() + ", ";
    outfitString += "Tops: ";
    for (int i = 0; i < topsCount; i++) {
      outfitString += tops[i]->getName();
      if (i < topsCount - 1) {
        outfitString += ", ";
      }
    }
    outfitString += ", ";
    outfitString += "Pants: " + pants->getName() + ", ";
    outfitString += "Shoes: " + shoes->getName();
    return outfitString;
  }
};

#endif