
// Libraries
#include <Adafruit_BMP280.h>
#include <DHT.h>
#include <LiquidCrystal_I2C.h>
#include <Wire.h>

// Constants
#define DHTPIN 8           // what pin we're connected to
#define DHTTYPE DHT22      // DHT 22
DHT dht(DHTPIN, DHTTYPE);  //// Initialize DHT sensor for normal 16mhz Arduino
Adafruit_BMP280 bmp;
Adafruit_Sensor *bmp_temp = bmp.getTemperatureSensor();
Adafruit_Sensor *bmp_pressure = bmp.getPressureSensor();
LiquidCrystal_I2C lcd(0x27, 16,
                      2);  // set the LCD address to 0x27 for a 16 chars and 2
                           // line display,Pls check your lcd.
                           // I2C
#include "Garment.h"

int h;  // Stores humidity value
int t;  // Stores temperature value
int p;

Headwear headwears[] = {Headwear("Black Cap", -10, 50, 2, true),
                        Headwear("Grey Beanie", -25, 15, 15, true)};

Top tops[] = {Top("Thick Black Jacket", -30, 5, 17.5, true),
              Top("Green Jacket", 0, 20, 10, false),
              Top("Black T-shirt", 15, 37.5, 5, false),
              Top("White T-shirt", 15, 32, 8, false),
              Top("Grey Tank Top", 25, 45, 2, false),
              Top("Rain Jacket", 5, 30, 8, true),
              Top("Grey Plaid Shirt", 5, 25, 5, false)};

Pants pants[] = {Pants("Black Pants", 5, 25, 10, true),
                 Pants("Blue Jeans", 5, 25, 12, true),
                 Pants("White Shorts", 25, 45, 5, true)};

Shoes shoes[] = {Shoes("Black Sneakers", -25, 25, 7, true),
                 Shoes("White Sneakers", 5, 40, 5, false),
                 Shoes("Blue Sneakers", 5, 40, 5, true)};

Outfit outfit = Outfit();
void chooseOutfit();

bool isInArray(int value, int *array, int arrayLength) {
  for (int i = 0; i < arrayLength; i++) {
    if (array[i] == value) {
      return true;
    }
  }
  return false;
}

void setup() {
  Serial.begin(9600);

  if (!bmp.begin()) {
    Serial.println(F("Could not find a valid BMP280 sensor, check wiring!"));
  }

  dht.begin();
  lcd.init();       // initialize the lcd
  lcd.backlight();  // open the backlight

  bmp.setSampling(Adafruit_BMP280::MODE_NORMAL,     /* Operating Mode. */
                  Adafruit_BMP280::SAMPLING_X2,     /* Temp. oversampling */
                  Adafruit_BMP280::SAMPLING_X16,    /* Pressure oversampling
                                                     */
                  Adafruit_BMP280::FILTER_X16,      /* Filtering. */
                  Adafruit_BMP280::STANDBY_MS_500); /* Standby time. */
}

void loop() {
  chooseOutfit();
  Serial.println(outfit.toString());

  h = dht.readHumidity();
  t = dht.readTemperature();

  sensors_event_t temp_event, pressure_event;
  bmp_temp->getEvent(&temp_event);
  bmp_pressure->getEvent(&pressure_event);
  p = pressure_event.pressure;

  char secondRow[17];
  snprintf(secondRow, sizeof(secondRow), "T:%dC H:%d%% P:%dhPa", t, h, p);
  // snprintf(secondRow, sizeof(secondRow), "T:%dC H:%d%%", t, h);

  String message = outfit.toString();
  int messageLength = message.length();
  int lcdWidth = 16;  // LCD width

  for (int i = 0; i < messageLength - lcdWidth + 1; i++) {
    String toDisplay = message.substring(i, i + lcdWidth);
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print(toDisplay);
    lcd.setCursor(0, 1);
    lcd.print(secondRow);
    delay(350);
  }
}

void chooseOutfit() {
  int temp = dht.readTemperature();
  int humidity = dht.readHumidity();
  bool badWeather = false;
  // How much warmer you want the outfit to be
  int relativeFeel = 2;
  // Maximum warmth you want relative to the temperature
  int maxWarmth = 15;
  // reset on the day
  outfit.topsCount = 0;

  // Headwear selection
  int headwearIndex = -1;
  for (int i = 0; i < sizeof(headwears) / sizeof(headwears[0]); i++) {
    if (headwears[i].isSuitable(temp + relativeFeel) &&
        headwears[i].weather_proof) {
      outfit.headwear = &headwears[i];
      headwearIndex = i;
      break;
    }
  }
  if (outfit.headwear == nullptr) {
    int randomIndex = random(sizeof(headwears) / sizeof(headwears[0]));
    outfit.headwear = &headwears[randomIndex];
    headwearIndex = randomIndex;
  }

  // Top Selection
  int top_feel = 0;
  int topIndeces[5] = {-1, -1, -1, -1, -1};
  bool selectedTops[sizeof(tops) / sizeof(tops[0])] = {false};

  if (badWeather) {
    for (int i = 0; i < sizeof(tops) / sizeof(tops[0]); i++) {
      if (!selectedTops[i] &&
          tops[i].isSuitable(temp + relativeFeel + top_feel) &&
          tops[i].weather_proof) {
        outfit.addTop(&tops[i]);
        top_feel += tops[i].added_warmth;
        topIndeces[0] = i;
        selectedTops[i] = true;
        break;
      }
    }
    if (outfit.topsCount == 0) {
      while (true) {
        int randomIndex = random(sizeof(tops) / sizeof(tops[0]));
        if (!selectedTops[randomIndex]) {
          outfit.addTop(&tops[randomIndex]);
          top_feel += tops[randomIndex].added_warmth;
          topIndeces[0] = randomIndex;
          selectedTops[randomIndex] = true;
          break;
        }
      }
    }
  }
  while (relativeFeel + top_feel < maxWarmth) {
    int minWarmth = 100;
    int minIndex = -1;
    for (int i = 0; i < sizeof(tops) / sizeof(tops[0]); i++) {
      if (!selectedTops[i] && tops[i].added_warmth < minWarmth &&
          tops[i].isSuitable(temp + relativeFeel + top_feel)) {
        minWarmth = tops[i].added_warmth;
        minIndex = i;
      }
    }
    if (minIndex != -1) {
      outfit.addTop(&tops[minIndex]);
      top_feel += tops[minIndex].added_warmth;
      topIndeces[outfit.topsCount - 1] = minIndex;
      selectedTops[minIndex] = true;
    } else {
      break;
    }
  }
  // Pants Selection
  for (int i = 0; i < sizeof(pants) / sizeof(pants[0]); i++) {
    if (pants[i].isSuitable(temp + relativeFeel) && pants[i].weather_proof) {
      outfit.pants = &pants[i];
      break;
    }
  }
  if (outfit.pants == nullptr) {
    int randomIndex = random(sizeof(pants) / sizeof(pants[0]));
    outfit.pants = &pants[randomIndex];
  }

  // Shoes Selection
  for (int i = 0; i < sizeof(shoes) / sizeof(shoes[0]); i++) {
    if (shoes[i].isSuitable(temp + relativeFeel) && shoes[i].weather_proof) {
      outfit.shoes = &shoes[i];
      break;
    }
  }
  if (outfit.shoes == nullptr) {
    int randomIndex = random(sizeof(shoes) / sizeof(shoes[0]));
    outfit.shoes = &shoes[randomIndex];
  }
}